#include "rusinluk_dip_incl.h"

CWin::CWin(int id, Gate *gate) :
	id(id),
	gateB(gate),
	gateE(gate),
	gateN(1)
{
}

CWin::~CWin() {
	while (gateB != NULL) {
		Gate *x = gateB;
		gateB = gateB->next;
		delete x;
	}
	removeEdges();
}

int CWin::getId() const {
	return id;
}

Gate *CWin::getGates() const {
	return gateB;
}

int CWin::getGateN() const {
	return gateN;
}

void CWin::mergeIO(const CWin *x, const TrieStor<int> &primOuts, TrieStor<CWin *> &neighbors,
	TrieStor<int> &workIns, TrieStor<int> &workOuts, TrieStor<int> &workIsect
) const {
	getNeighbors(x, neighbors);
	workIns = ins;
	workIns += x->ins;
	workOuts = outs;
	workOuts += x->outs;
	workIsect.isect(workIns, workOuts);
	workIns -= workIsect;
	for (int i = workIsect.size() - 1; i >= 0; i--) {
		int item = workIsect[i];
		if (primOuts.find(item)) {
			workIsect.remove(item);
			continue;
		}
		for (unsigned j = 0; j < neighbors.size(); j++) {
			if (neighbors[j]->ins.find(item)) {
				workIsect.remove(item);
				break;
			}
		}
	}
	workOuts -= workIsect;
}

void CWin::merge(CWin *x, const TrieStor<int> &primOuts, TrieStor<CWin *> &neighbors,
	TrieStor<int> &workIns, TrieStor<int> &workOuts, TrieStor<int> &workIsect
) {
	mergeIO(x, primOuts, neighbors, workIns, workOuts, workIsect);
	ins = workIns;
	outs = workOuts;
	gateE->next = x->gateB;
	gateE = x->gateE;
	x->gateB = NULL;
	gateN += x->gateN;
}

void CWin::removeEdges() {
	for (int i = edges.size() - 1; i >= 0; i--) {
		CWinEdge *edge = edges[i];
		edges.remove(edge);
		edge->adr(this)->edges.remove(edge); // neighbor's list
		delete edge;
	}
}

void CWin::getNeighbors(const CWin *x, TrieStor<CWin *> &res) const {
	res.reset();
	for (unsigned j = 0; j < edges.size(); j++) {
		res.add(edges[j]->adr(this));
	}
	for (unsigned j = 0; j < x->edges.size(); j++) {
		res.add(x->edges[j]->adr(x));
	}
	const CWin *me = this;
	res.remove(const_cast<CWin *>(me));
	res.remove(const_cast<CWin *>(x));
}
