#include "rusinluk_dip_incl.h"

Params::Params() :
	fileI(""),
	fileO(""),
	cwinN(ARGD_CWIN),
	wgCwinI(ARGD_CWIN_WG1),
	wgCwinO(ARGD_CWIN_WG2),
	wgCwinG(ARGD_CWIN_WG3),
	wgRwinG(ARGD_RWIN_WG1),
	wgRwinGI(ARGD_RWIN_WG2),
	wgRwinL(ARGD_RWIN_WG3),
	methods(""),
	force(false),
	rand(true),
	clear(true),
	log(false),
	logW(true)
{
}

bool Params::load(int argc, char *argv[]) {
	int i = 1;
	while (i < argc) {
		if (!strcmp(argv[i], ARG_OUTFILE)) {
			if (i++ + 1 >= argc) {
				return false;
			}
			fileO = argv[i++];
			continue;
		}
		if (!strcmp(argv[i], ARG_CWIN)) {
			if (i++ + 1 >= argc) {
				return false;
			}
			cwinN = atoi(argv[i++]);
			continue;
		}
		if (!strcmp(argv[i], ARG_CWIN_WG)) {
			if (i++ + 3 >= argc) {
				return false;
			}
			wgCwinI = atof(argv[i++]);
			wgCwinO = atof(argv[i++]);
			wgCwinG = atof(argv[i++]);
			continue;
		}
		if (!strcmp(argv[i], ARG_RWIN_WG)) {
			if (i++ + 3 >= argc) {
				return false;
			}
			wgRwinG = atof(argv[i++]);
			wgRwinGI = atof(argv[i++]);
			wgRwinL = atof(argv[i++]);
			continue;
		}
		if (!strcmp(argv[i], ARG_METHODS)) {
			if (i++ + 1 >= argc) {
				return false;
			}
			methods = argv[i++];
			continue;
		}
		if (!strcmp(argv[i], ARG_FORCE)) {
			i++;
			force = true;
			continue;
		}
		if (!strcmp(argv[i], ARG_RSEED)) {
			if (i++ + 2 >= argc) {
				return false;
			}
			rand = false;
			rseed1 = atoi(argv[i++]);
			rseed2 = atoi(argv[i++]);
			continue;
		}
		if (!strcmp(argv[i], ARG_NOT_CLEAR)) {
			i++;
			clear = false;
			continue;
		}
		if (!strcmp(argv[i], ARG_LOG)) {
			i++;
			log = true;
			continue;
		}
		if (!strcmp(argv[i], ARG_NO_WARNINGS)) {
			i++;
			logW = false;
			continue;
		}
		if (fileI.size()) {
			return false;
		}
		fileI = argv[i++];
	}
	if (!fileI.size()) {
		return false;
	}
	check();
	return true;
}

void Params::print() const {
	logRow("Params:");
	string s("  ");
	logRow(s + ARGI_INFILE + " = " + fileI);
	logRow(s + ARGI_OUTFILE + " = " + fileO);
	logRow(s + ARGI_CWIN + " = " + (cwinN > 0 ? toString(cwinN) : "don't make"));
	logRow(s + ARGI_CWIN_WG + " = " + toString(wgCwinI) + " (inputs), " +
		toString(wgCwinO) + " (outputs), " + toString(wgCwinG) + " (gates)");
	logRow(s + ARGI_RWIN_WG + " = " + toString(wgRwinG) + " (gates), " +
		toString(wgRwinGI) + " (gate inputs), " + toString(wgRwinL) + " (levels)");
	logRow(s + ARGI_METHODS + " = " + (methods.size() ? methods : "no"));
	logRow(s + ARGI_FORCE + " = " + (force ? "yes" : "no"));
	logRow(s + ARGI_RSEED + " = " + (rand ? "random" : toString(rseed1) + ", " + toString(rseed2)));
	logRow(s + ARGI_NOT_CLEAR + " = " + (clear ? "no" : "yes"));
	logRow(s + ARGI_LOG + " = " + (log ? "yes" : "no"));
	logRow(s + ARGI_NO_WARNINGS + " = " + (logW ? "no" : "yes"));
}

void Params::printArgs() const {
	logRow("Args:");
	string s("  ");
	logRow(s + "STRING -- " + ARGI_INFILE);
	logRow(s + "[" + ARG_OUTFILE + " STRING] -- " + ARGI_OUTFILE);
	logRow(s + "[" + ARG_CWIN + " INT] -- " + ARGI_CWIN);
	logRow(s + "[" + ARG_CWIN_WG + " FLOAT FLOAT FLOAT] -- " + ARGI_CWIN_WG);
	logRow(s + "[" + ARG_RWIN_WG + " FLOAT FLOAT FLOAT] -- " + ARGI_RWIN_WG);
	logRow(s + "[" + ARG_METHODS + " STRING] -- " + ARGI_METHODS);
	logRow(s + "[" + ARG_FORCE + "] -- " + ARGI_FORCE);
	logRow(s + "[" + ARG_RSEED + " INT INT] -- " + ARGI_RSEED);
	logRow(s + "[" + ARG_NOT_CLEAR + "] -- " + ARGI_NOT_CLEAR);
	logRow(s + "[" + ARG_LOG + "] -- " + ARGI_LOG);
	logRow(s + "[" + ARG_NO_WARNINGS + "] -- " + ARGI_NO_WARNINGS);
}

void Params::check() {
	if (!fileO.size()) {
		fileO = fileI;
		fileSuffix(fileO, FILE_OUTPUT);
	}
	if (cwinN < 0) {
		cwinN = 0;
	}
	if (!checkWg(wgCwinI, wgCwinO, wgCwinG)) {
		wgCwinI = ARGD_CWIN_WG1;
		wgCwinO = ARGD_CWIN_WG2;
		wgCwinG = ARGD_CWIN_WG3;
	}
	if (!checkWg(wgRwinG, wgRwinGI, wgRwinL)) {
		wgRwinG = ARGD_RWIN_WG1;
		wgRwinGI = ARGD_RWIN_WG2;
		wgRwinL = ARGD_RWIN_WG3;
	}
}

bool Params::checkWg(double wg1, double wg2, double wg3) {
	return wg1 >= 0 && wg2 >= 0 && wg3 >= 0 && (wg1 + wg2 + wg3) > 0;
}
