#include "rusinluk_dip_incl.h"

Scheduler::Scheduler(int cwinN, int methodN) :
	Storable(sizeof(Scheduler) + sizeof(int) * cwinN),
	cwinN(cwinN),
	methodN(methodN)
{
	setPtrs();
	reset();
}

void Scheduler::reset() {
	for (int i = 0; i < cwinN; i++) {
		counts[i] = methodN;
	}
	countG = cwinN * (methodN + 1);
	nextI = 0;
	done = false;
}

void Scheduler::submitWork(RWork &work) {
	if (work.isValid()) {
		counts[work.cwinI]--;
		done = !--countG;
	}
	if (mpiRank() != BOSS_RANK || mpiProcN() == 1 || countG == 1) {
		work.setEval();
		if (work.isValid() && !counts[work.cwinI]) {
			counts[work.cwinI] = -1; // given
			return;
		}
		for (int i = 0; i < cwinN; i++) {
			if (!counts[i]) {
				counts[i] = -1;
				work.cwinI = i;
				return;
			}
		}
	}
	if (nextI >= cwinN * methodN) {
		work.setInvalid();
		return;
	}
	work.cwinI = nextI / methodN;
	work.methodI = nextI % methodN;
	nextI++;
}

bool Scheduler::isDone() {
	return done;
}

void Scheduler::save() {
	memcpy(getBuf(), this, sizeof(Scheduler));
}

void Scheduler::setPtrs() {
	counts = (int *)((Scheduler *)getBuf() + 1);
}

void Scheduler::load_0() {
	memcpy(this, getBuf(), sizeof(Scheduler));
}
