#include "rusinluk_dip_incl.h"

SharedMem::SharedMem(int size) {
	if (size < 0) {
		size = 0;
	}
	if (mpiRank() == BOSS_RANK) {
		MPI_Alloc_mem(size, MPI_INFO_NULL, &buf);
		MPI_Win_create(buf, size, 1, MPI_INFO_NULL, MPI_COMM_WORLD, &win);
	}
	else {
		MPI_Win_create(NULL, 0, 1, MPI_INFO_NULL, MPI_COMM_WORLD, &win);
	}
	mpiBarrier();
}

SharedMem::~SharedMem() {
	MPI_Win_free(&win); // also as a barrier
	if (mpiRank() == BOSS_RANK) {
		MPI_Free_mem(buf);
	}
}

void SharedMem::lock() {
	MPI_Win_lock(MPI_LOCK_SHARED, BOSS_RANK, 0, win);
}

void SharedMem::unlock() {
	MPI_Win_unlock(BOSS_RANK, win);
}

void SharedMem::use() {
	lock();
	unlock();
}
