#include "rusinluk_dip_incl.h"

string toString(double x) {
	stringstream ss;
	ss << setprecision(numeric_limits<double>::digits10 + 1) << x;
	return ss.str();
}

void logMe(string s, bool endL) {
	string file = "rusinluk_log" + toString(mpiRank()) + ".txt";
	ofstream ofs;
	ofs.open(file.c_str(), ios_base::app);
	ofs << s;
	cout << s;
	if (endL) {
		ofs << endl;
		cout << endl;
	}
}

void logRow(string s) {
	logMe(s, true);
}

void logErr(string s) {
	logRow(s);
	mpiDelay(WAIT_ERROR); // for done logging on other processes
	mpiAbort();
}

void logErr(string errT, string errS) {
	logErr(errT + ": " + errS);
}

void logTime(string msg, double time0) {
	logRow(msg + " -- " + toString(MPI_Wtime() - time0) + " s");
}

void parseNames(const string &s, vector<string> &res) {
	res.clear();
	size_t i = 0; // after previous space
	bool end = false;
	while (1) {
		size_t j = s.find_first_of(' ', i);
		if (j == string::npos) {
			j = s.size();
			end = true;
		}
		if (j > i) {
			res.push_back(s.substr(i, j - i)); // only non-empty names
		}
		if (end) {
			return;
		}
		i = j + 1;
	}
}

bool replaceStr(string &s, const string &s0, const string &s1, int occurI) {
	size_t pos = 0;
	if (occurI > 0) {
		for (int i = 0; i < occurI; i++) {
			if ((pos = s.find(s0, pos)) == string::npos) {
				return false;
			}
			pos += s0.size();
		}
		s.replace(pos - s0.size(), s0.size(), s1);
		return true;
	}
	if (occurI == 0) {
		bool found = false;
		while ((pos = s.find(s0, pos)) != string::npos) {
			s.replace(pos, s0.size(), s1);
			pos += s1.size();
			found = true;
		}
		return found;
	}
	int n = 0;
	while ((pos = s.find(s0, pos)) != string::npos) {
		pos += s0.size();
		n++;
	}
	pos = 0;
	for (int i = 0; i < n; i++) {
		pos = s.find(s0, pos);
		if (i == n + occurI) {
			s.replace(pos, s0.size(), s1);
			return true;
		}
		pos += s0.size();
	}
	return false;
}

void fileSuffix(string &file, string x) {
	string dot = ".";
	if (file.find(dot) == string::npos) {
		file += x;
		return;
	}
	x += dot;
	replaceStr(file, dot, x, -1);
}

void fileTest(const string &file) {
	ifstream ifs(file.c_str(), ios_base::in);
	if (!ifs.is_open()) {
		logErr(ERR_FOPEN, file + " expected to use.");
	}
}

void ifsOpen(ifstream &ifs, const string &file, ios_base::openmode mode) {
	ifs.open(file.c_str(), mode);
	if (!ifs.is_open()) {
		logErr(ERR_FOPEN, file + " failed to open for reading.");
	}
}

void ofsOpen(ofstream &ofs, const string &file, ios_base::openmode mode) {
	ofs.open(file.c_str(), mode);
	if (!ofs.is_open()) {
		logErr(ERR_FOPEN, file + " failed to open for writing.");
	}
}

void ifsOpenWait(ifstream &ifs, const string &file, ios_base::openmode mode, double t, double coef) {
	while (1) {
		ifs.open(file.c_str(), mode);
			// possible opening collisions on the system
		if (ifs.is_open()) {
			break;
		}
		mpiDelay(t);
		t *= coef;
		if (t > WAIT_MAX) {
			t = WAIT_MAX;
		}
	}
}

void fileCopy(const string &fileI, const string &fileO) {
	ifstream ifs;
	ifsOpen(ifs, fileI, ios_base::in | ios_base::binary);
	ofstream ofs;
	ofsOpen(ofs, fileO, ios_base::out | ios_base::binary);
	ofs << ifs.rdbuf();
}

bool getFLine(ifstream &ifs, string &s) {
	if (!getline(ifs, s)) {
		return false;
	}
	if (s.size() && s[s.size() - 1] == '\r') {
		s = s.substr(0, s.size() - 1);
	}
	return true;
}

bool getBlifLine(ifstream &ifs, string &s) {
	if (!getFLine(ifs, s)) {
		return false;
	}
	if (!s.size()) {
		return true;
	}
	string str;
	while (s[s.size() - 1] == '\\') {
		s[s.size() - 1] = ' '; // ensure a separator
		if (!getFLine(ifs, str)) {
			break;
		}
		s += str;
	}
	size_t pos = s.find(BLIF_COMMENT);
	if (pos != string::npos) {
		s = s.substr(0, pos);
	}
	return true;
}

int mpiProcN() {
	int res;
	MPI_Comm_size(MPI_COMM_WORLD, &res);
	return res;
}

int mpiRank() {
	int res;
	MPI_Comm_rank(MPI_COMM_WORLD, &res);
	return res;
}

void mpiAbort() {
	MPI_Abort(MPI_COMM_WORLD, 1);
}

void mpiDelay(double t) {
	double time0 = MPI_Wtime();
	while (MPI_Wtime() - time0 < t);
}

void waitPhase(double t) {
	mpiDelay(mpiRank() * t / mpiProcN());
}

void mpiBarrier() {
	MPI_Barrier(MPI_COMM_WORLD);
}

template<typename T, typename Q>
Q *dynCast(T *x) {
	Q *res = dynamic_cast<Q *>(x);
	if (res == NULL && x != NULL) {
		string nameX(typeid(*x).name());
		string nameQ(typeid(Q).name());
		logErr("Error: " + nameX + " object cannot be pointed by *" + nameQ);
	}
	return res;
}
	template CWinEdge *dynCast<Heapable, CWinEdge>(Heapable *);
